==> xwiki-web-gwt-wysiwyg-client-2.3-rc-1.jar: org/xwiki/gwt/wysiwyg/client/plugin/link/ui/AbstractExplorerWizardStep.java <==
/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.xwiki.gwt.wysiwyg.client.plugin.link.ui;

import org.xwiki.gwt.user.client.ui.VerticalResizePanel;
import org.xwiki.gwt.user.client.ui.wizard.NavigationListener;
import org.xwiki.gwt.user.client.ui.wizard.NavigationListenerCollection;
import org.xwiki.gwt.user.client.ui.wizard.SourcesNavigationEvents;
import org.xwiki.gwt.user.client.ui.wizard.NavigationListener.NavigationDirection;
import org.xwiki.gwt.wysiwyg.client.plugin.link.LinkConfig;
import org.xwiki.gwt.wysiwyg.client.widget.explorer.XWikiExplorer;
import org.xwiki.gwt.wysiwyg.client.widget.explorer.ds.WikiDataSource;
import org.xwiki.gwt.wysiwyg.client.widget.wizard.util.AbstractSelectorWizardStep;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;
import com.smartgwt.client.widgets.events.KeyPressEvent;
import com.smartgwt.client.widgets.events.KeyPressHandler;
import com.smartgwt.client.widgets.grid.events.RecordDoubleClickEvent;
import com.smartgwt.client.widgets.grid.events.RecordDoubleClickHandler;

/**
 * Wizard step to provide an interface to selecting a wiki resource, using an {@link XWikiExplorer}.
 * 
 * @version $Id: AbstractExplorerWizardStep.java 26786 2010-02-07 16:46:17Z mflorea $
 */
public abstract class AbstractExplorerWizardStep extends AbstractSelectorWizardStep<LinkConfig> implements
    SourcesNavigationEvents, RecordDoubleClickHandler, KeyPressHandler
{
    /**
     * The style of the fields under error.
     */
    protected static final String FIELD_ERROR_STYLE = "xErrorField";

    /**
     * The xwiki tree explorer, used to select the page or file to link to.
     */
    private XWikiExplorer explorer;

    /**
     * The panel to hold the xwiki explorer.
     */
    private final VerticalResizePanel mainPanel = new VerticalResizePanel();

    /**
     * The label to display the error on submission of the wizard step form.
     */
    private final Label errorLabel = new Label();

    /**
     * The collection of listeners to launch navigation events to, when an item in the tree displayed by this step is
     * double clicked.
     */
    private NavigationListenerCollection listeners = new NavigationListenerCollection();

    /**
     * Builds a {@link AbstractExplorerWizardStep} from the passed settings.
     * 
     * @param addPage specifies whether the wiki explorer should show the option to add a page
     * @param showAttachments specifies whether the wiki explorer should show the attached files for pages
     * @param addAttachments specifies whether the wiki explorer should show the option to add an attachment
     * @param defaultSelection the default selection of the wiki explorer displayed by this step
     */
    public AbstractExplorerWizardStep(boolean addPage, boolean showAttachments, boolean addAttachments,
        String defaultSelection)
    {
        this(addPage, showAttachments, addAttachments, defaultSelection, 455, 305);
    }

    /**
     * Builds a {@link AbstractExplorerWizardStep} from the passed settings, with parameters for size. <br />
     * FIXME: remove the size parameters when the explorer will be correctly sizable from CSS.
     * 
     * @param addPage specifies whether the wiki explorer should show the option to add a page
     * @param showAttachments specifies whether the wiki explorer should show the attached files for pages
     * @param addAttachments specifies whether the wiki explorer should show the option to add an attachment
     * @param defaultSelection the default selection of the wiki explorer displayed by this step
     * @param width explorer width in pixels
     * @param height explorer height in pixels
     */
    protected AbstractExplorerWizardStep(boolean addPage, boolean showAttachments, boolean addAttachments,
        String defaultSelection, int width, int height)
    {
        explorer = new XWikiExplorer();
        explorer.setDisplayLinks(false);
        // display the new page option
        explorer.setDisplayAddPage(addPage);
        explorer.setDisplayAddPageOnTop(true);
        // no attachments here
        explorer.setDisplayAttachments(showAttachments);
        explorer.setDisplayAddAttachment(showAttachments && addAttachments);
        explorer.setDisplayAddAttachmentOnTop(true);
        explorer.setDisplayAttachmentsWhenEmpty(showAttachments && addAttachments);
        String sizeUnit = "px";
        explorer.setWidth(width + sizeUnit);
        explorer.setHeight(height + sizeUnit);
        WikiDataSource ds = new WikiDataSource();
        explorer.setDataSource(ds);
        explorer.setDefaultValue(defaultSelection);
        // FIXME: this is somewhat implementation specific, explorer.getElement returns the explorer wrapper while
        // explorer.addStyleName() actually sets the style on the inner tree. We need the style applied on the outer
        // wrapper.
        explorer.getElement().setClassName(explorer.getElement().getClassName() + " xExplorer");

        explorer.addRecordDoubleClickHandler(this);
        explorer.addKeyPressHandler(this);

        // create a label with the help for this step
        Label helpLabel = new Label();
        helpLabel.addStyleName("xHelpLabel");
        helpLabel.setText(getHelpLabelText());
        mainPanel.add(helpLabel);

        errorLabel.setText(getDefaultErrorText());
        errorLabel.setVisible(false);
        errorLabel.addStyleName("xErrorMsg");
        mainPanel.add(errorLabel);

        mainPanel.addStyleName("xExplorerPanel");
        // we need to add the explorer in a wrapper, since the explorer creates its own wrapper around and adds the
        // input to that wrapper. We use this panel to have a reference to the _whole_ generated UI, since the explorer
        // reference would point only to the grid inside.
        mainPanel.add(explorer);
        mainPanel.setExpandingWidget(explorer, true);
    }

    /**
     * @return the help message for this explorer step, to be displayed on top of the explorer tree
     */
    protected abstract String getHelpLabelText();

    /**
     * @return the default error message for this wizard step form
     */
    protected abstract String getDefaultErrorText();

    /**
     * Invalidates the cache on the explorer, so that it will be reloaded on next display. To be used to request an
     * update of the tree when new data is added to it.
     */
    protected void invalidateExplorerData()
    {
        // let's be silently safe about it, no calling function should fail because of this, at least for the moment
        try {
            explorer.invalidateCache();
        } catch (Exception e) {
            // nothing
        }
    }

    /**
     * {@inheritDoc}
     */
    public Widget display()
    {
        return mainPanel;
    }

    /**
     * @return the wiki explorer used by this selector
     */
    public XWikiExplorer getExplorer()
    {
        return explorer;
    }

    /**
     * @return the error label
     */
    public Label getErrorLabel()
    {
        return errorLabel;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void init(Object data, AsyncCallback< ? > cb)
    {
        hideError();
        super.init(data, cb);
    }

    /**
     * Displays the passed error message and markers in this wizard step.
     * 
     * @param errorMessage the error message to print
     */
    protected void displayError(String errorMessage)
    {
        errorLabel.setText(errorMessage);
        errorLabel.setVisible(true);
        // set the class at the wrapper level, the element of this explorer
        if (!explorer.getElement().getClassName().contains(FIELD_ERROR_STYLE)) {
            explorer.getElement().setClassName(explorer.getElement().getClassName() + " " + FIELD_ERROR_STYLE);
        }
        mainPanel.refreshHeights();
    }

    /**
     * Hides the error messages and markers in this wizard step.
     */
    protected void hideError()
    {
        errorLabel.setVisible(false);
        // remove the class from the wrapper level, the element of this explorer
        String boundary = "\\b";
        explorer.getElement().setClassName(
            explorer.getElement().getClassName().replaceAll(boundary + FIELD_ERROR_STYLE + boundary, ""));
        mainPanel.refreshHeights();
    }

    /**
     * {@inheritDoc}
     */
    public void addNavigationListener(NavigationListener listener)
    {
        listeners.add(listener);
    }

    /**
     * {@inheritDoc}
     */
    public void removeNavigationListener(NavigationListener listener)
    {
        listeners.remove(listener);
    }

    /**
     * {@inheritDoc}
     */
    public void onRecordDoubleClick(RecordDoubleClickEvent event)
    {
        listeners.fireNavigationEvent(NavigationDirection.NEXT);
    }

    /**
     * {@inheritDoc}
     */
    public void onKeyPress(KeyPressEvent event)
    {
        // :)
        if ("Enter".equals(event.getKeyName())) {
            listeners.fireNavigationEvent(NavigationDirection.NEXT);
        }
    }
}

==> xwiki-web-gwt-wysiwyg-client-2.4.1.jar: org/xwiki/gwt/wysiwyg/client/plugin/link/ui/AbstractExplorerWizardStep.java <==
/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.xwiki.gwt.wysiwyg.client.plugin.link.ui;

import org.xwiki.gwt.user.client.StringUtils;
import org.xwiki.gwt.user.client.ui.VerticalResizePanel;
import org.xwiki.gwt.user.client.ui.wizard.NavigationListener;
import org.xwiki.gwt.user.client.ui.wizard.NavigationListenerCollection;
import org.xwiki.gwt.user.client.ui.wizard.SourcesNavigationEvents;
import org.xwiki.gwt.user.client.ui.wizard.NavigationListener.NavigationDirection;
import org.xwiki.gwt.wysiwyg.client.plugin.link.LinkConfig;
import org.xwiki.gwt.wysiwyg.client.widget.explorer.XWikiExplorer;
import org.xwiki.gwt.wysiwyg.client.widget.explorer.ds.WikiDataSource;
import org.xwiki.gwt.wysiwyg.client.widget.wizard.util.AbstractSelectorWizardStep;
import org.xwiki.gwt.wysiwyg.client.wiki.EntityConfig;
import org.xwiki.gwt.wysiwyg.client.wiki.EntityLink;
import org.xwiki.gwt.wysiwyg.client.wiki.EntityReference;
import org.xwiki.gwt.wysiwyg.client.wiki.WikiServiceAsync;
import org.xwiki.gwt.wysiwyg.client.wiki.EntityReference.EntityType;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;
import com.smartgwt.client.widgets.events.KeyPressEvent;
import com.smartgwt.client.widgets.events.KeyPressHandler;
import com.smartgwt.client.widgets.grid.events.RecordDoubleClickEvent;
import com.smartgwt.client.widgets.grid.events.RecordDoubleClickHandler;

/**
 * Wizard step to provide an interface to selecting a wiki resource, using an {@link XWikiExplorer}.
 * 
 * @version $Id: AbstractExplorerWizardStep.java 28844 2010-05-12 13:22:39Z mflorea $
 */
public abstract class AbstractExplorerWizardStep extends AbstractSelectorWizardStep<EntityLink<LinkConfig>> implements
    SourcesNavigationEvents, RecordDoubleClickHandler, KeyPressHandler
{
    /**
     * The style of the fields under error.
     */
    protected static final String FIELD_ERROR_STYLE = "xErrorField";

    /**
     * The xwiki tree explorer, used to select the page or file to link to.
     */
    private final XWikiExplorer explorer = new XWikiExplorer();

    /**
     * The panel to hold the xwiki explorer.
     */
    private final VerticalResizePanel mainPanel = new VerticalResizePanel();

    /**
     * The label to display the error on submission of the wizard step form.
     */
    private final Label errorLabel = new Label();

    /**
     * The help message displayed on the top of the explorer tree.
     */
    private final Label helpLabel = new Label();

    /**
     * The collection of listeners to launch navigation events to, when an item in the tree displayed by this step is
     * double clicked.
     */
    private final NavigationListenerCollection listeners = new NavigationListenerCollection();

    /**
     * The service used to parse and serialize entity references.
     */
    private final WikiServiceAsync wikiService;

    /**
     * The step title.
     */
    private String stepTitle;

    /**
     * Builds a {@link AbstractExplorerWizardStep} from the passed settings.
     * 
     * @param wikiService the service to be used to parse and serialize entity references
     * @param addPage specifies whether the wiki explorer should show the option to add a page
     * @param showAttachments specifies whether the wiki explorer should show the attached files for pages
     * @param addAttachments specifies whether the wiki explorer should show the option to add an attachment
     */
    public AbstractExplorerWizardStep(WikiServiceAsync wikiService, boolean addPage, boolean showAttachments,
        boolean addAttachments)
    {
        this(wikiService, addPage, showAttachments, addAttachments, 455, 305);
    }

    /**
     * Builds a {@link AbstractExplorerWizardStep} from the passed settings, with parameters for size. <br />
     * FIXME: remove the size parameters when the explorer will be correctly sizable from CSS.
     * 
     * @param wikiService the service to be used to parse and serialize entity references
     * @param addPage specifies whether the wiki explorer should show the option to add a page
     * @param showAttachments specifies whether the wiki explorer should show the attached files for pages
     * @param addAttachments specifies whether the wiki explorer should show the option to add an attachment
     * @param width explorer width in pixels
     * @param height explorer height in pixels
     */
    protected AbstractExplorerWizardStep(WikiServiceAsync wikiService, boolean addPage, boolean showAttachments,
        boolean addAttachments, int width, int height)
    {
        this.wikiService = wikiService;

        explorer.setDisplayLinks(false);
        // display the new page option
        explorer.setDisplayAddPage(addPage);
        explorer.setDisplayAddPageOnTop(true);
        // no attachments here
        explorer.setDisplayAttachments(showAttachments);
        explorer.setDisplayAddAttachment(showAttachments && addAttachments);
        explorer.setDisplayAddAttachmentOnTop(true);
        explorer.setDisplayAttachmentsWhenEmpty(showAttachments && addAttachments);
        String sizeUnit = "px";
        explorer.setWidth(width + sizeUnit);
        explorer.setHeight(height + sizeUnit);
        WikiDataSource ds = new WikiDataSource();
        explorer.setDataSource(ds);
        // Select nothing by default.
        explorer.setDefaultValue("");
        // FIXME: this is somewhat implementation specific, explorer.getElement returns the explorer wrapper while
        // explorer.addStyleName() actually sets the style on the inner tree. We need the style applied on the outer
        // wrapper.
        explorer.getElement().setClassName(explorer.getElement().getClassName() + " xExplorer");

        explorer.addRecordDoubleClickHandler(this);
        explorer.addKeyPressHandler(this);

        helpLabel.addStyleName("xHelpLabel");
        mainPanel.add(helpLabel);

        errorLabel.setVisible(false);
        errorLabel.addStyleName("xErrorMsg");
        mainPanel.add(errorLabel);

        mainPanel.addStyleName("xExplorerPanel");
        // we need to add the explorer in a wrapper, since the explorer creates its own wrapper around and adds the
        // input to that wrapper. We use this panel to have a reference to the _whole_ generated UI, since the explorer
        // reference would point only to the grid inside.
        mainPanel.add(explorer);
        mainPanel.setExpandingWidget(explorer, true);
    }

    /**
     * Sets the help message to be displayed on top of the explorer tree.
     * 
     * @param helpLabelText the new help message
     */
    public void setHelpLabelText(String helpLabelText)
    {
        helpLabel.setText(helpLabelText);
    }

    /**
     * Invalidates the cache on the explorer, so that it will be reloaded on next display. To be used to request an
     * update of the tree when new data is added to it.
     */
    protected void invalidateExplorerData()
    {
        // let's be silently safe about it, no calling function should fail because of this, at least for the moment
        try {
            explorer.invalidateCache();
        } catch (Exception e) {
            // nothing
        }
    }

    /**
     * {@inheritDoc}
     */
    public Widget display()
    {
        return mainPanel;
    }

    /**
     * @return the wiki explorer used by this selector
     */
    public XWikiExplorer getExplorer()
    {
        return explorer;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void init(Object data, AsyncCallback< ? > cb)
    {
        hideError();
        super.init(data, cb);
    }

    /**
     * {@inheritDoc}
     * 
     * @see AbstractSelectorWizardStep#initializeSelection(AsyncCallback)
     */
    @Override
    protected void initializeSelection(AsyncCallback< ? > initCallback)
    {
        String reference = getData().getData().getReference();
        if (!StringUtils.isEmpty(reference)) {
            // Edit link (link reference is not empty).
            getExplorer().selectEntity(getData().getDestination(), null);
        } else if (StringUtils.isEmpty(getExplorer().getValue())) {
            // New link (no entity has been previously selected).
            getExplorer().selectEntity(getData().getOrigin(), getGroupAnchor(getData().getDestination().getType()));
        } // Else, preserve the current selection.
        super.initializeSelection(initCallback);
    }

    /**
     * @param entityType an entity type
     * @return the anchor that groups the entities with the specified type under their parent entity
     */
    private String getGroupAnchor(EntityType entityType)
    {
        return entityType == EntityType.ATTACHMENT || entityType == EntityType.IMAGE ? "Attachments" : null;
    }

    /**
     * Displays the passed error message and markers in this wizard step.
     * 
     * @param errorMessage the error message to print
     */
    protected void displayError(String errorMessage)
    {
        errorLabel.setText(errorMessage);
        errorLabel.setVisible(true);
        // set the class at the wrapper level, the element of this explorer
        if (!explorer.getElement().getClassName().contains(FIELD_ERROR_STYLE)) {
            explorer.getElement().setClassName(explorer.getElement().getClassName() + " " + FIELD_ERROR_STYLE);
        }
        mainPanel.refreshHeights();
    }

    /**
     * Hides the error messages and markers in this wizard step.
     */
    protected void hideError()
    {
        errorLabel.setVisible(false);
        // remove the class from the wrapper level, the element of this explorer
        String boundary = "\\b";
        explorer.getElement().setClassName(
            explorer.getElement().getClassName().replaceAll(boundary + FIELD_ERROR_STYLE + boundary, ""));
        mainPanel.refreshHeights();
    }

    /**
     * {@inheritDoc}
     */
    public void addNavigationListener(NavigationListener listener)
    {
        listeners.add(listener);
    }

    /**
     * {@inheritDoc}
     */
    public void removeNavigationListener(NavigationListener listener)
    {
        listeners.remove(listener);
    }

    /**
     * {@inheritDoc}
     */
    public void onRecordDoubleClick(RecordDoubleClickEvent event)
    {
        listeners.fireNavigationEvent(NavigationDirection.NEXT);
    }

    /**
     * {@inheritDoc}
     */
    public void onKeyPress(KeyPressEvent event)
    {
        // :)
        if ("Enter".equals(event.getKeyName())) {
            listeners.fireNavigationEvent(NavigationDirection.NEXT);
        }
    }

    /**
     * Updates the link configuration object based on the selected entity.
     * 
     * @param selectedEntityReference a reference to the selected entity
     * @param callback the object to be notified after the link configuration object has been updated
     */
    protected void updateLinkConfig(final EntityReference selectedEntityReference,
        final AsyncCallback<Boolean> callback)
    {
        wikiService.getEntityConfig(getData().getOrigin(), selectedEntityReference, new AsyncCallback<EntityConfig>()
        {
            public void onFailure(Throwable caught)
            {
                callback.onFailure(caught);
            }

            public void onSuccess(EntityConfig result)
            {
                getData().setDestination(selectedEntityReference.clone());
                getData().getData().setReference(result.getReference());
                getData().getData().setUrl(result.getUrl());
                callback.onSuccess(true);
            }
        });
    }

    /**
     * @return the service used to parse and serialize entity references
     */
    protected WikiServiceAsync getWikiService()
    {
        return wikiService;
    }

    /**
     * {@inheritDoc}
     * 
     * @see AbstractSelectorWizardStep#onCancel()
     */
    public void onCancel()
    {
        // Do nothing.
    }

    /**
     * {@inheritDoc}
     * 
     * @see AbstractSelectorWizardStep#getStepTitle()
     */
    public String getStepTitle()
    {
        return stepTitle;
    }

    /**
     * Sets the step title.
     * 
     * @param stepTitle the new step title
     */
    public void setStepTitle(String stepTitle)
    {
        this.stepTitle = stepTitle;
    }
}
